==> BaseModel-20190524.1622.tar.gz: BaseModel-20190524.1622/BaseModel/BaseModel.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/14 14:18
# @Author   : Peter
# @Site       : Model基类
# @File        : BaseModel.py
# @Software: PyCharm
from sqlalchemy import func, or_, distinct


class BaseModel:

    # 更新model
    @staticmethod
    def mergeModel(model, session, commit=True):
        session.merge(model)
        if commit:
            session.commit()

    # 添加model
    @staticmethod
    def addModel(model, session, commit=True):
        session.add(model)
        if commit:
            session.commit()

    # query
    @staticmethod
    def query(sql, session, *args, **kwargs):
        return session.execute(sql)

    # 通过id查询
    @classmethod
    def get(cls, _id, session):
        return session.query(cls).get(_id)

    # 查询所有
    @classmethod
    def getAll(cls, session):
        return session.query(cls).all()

    @classmethod
    def getDistinctK(cls, k, session, **kw):
        if kw.get("columns"):
            cls_colum = []
            for Colum in kw.get("columns", []):
                cls_colum.append(getattr(cls, Colum))
            return session.query(*cls_colum).distinct(getattr(cls, k)).order_by(kw.get("order", k)).all()
        else:
            return session.query(distinct(getattr(cls, k))).order_by(kw.get("order", "id")).all()

    # 查询所有
    @classmethod
    def getAllByCondition(cls, cond, session, relation="and", **kw):
        condition = list()
        for k, v in dict(cond).items():
            condition += [getattr(cls, k) == v]
        if relation == "and":
            return session.query(cls).filter(*condition).order_by("id").all()
        elif relation == "or":
            return session.query(cls).filter(or_(*condition)).order_by("id").all()
        else:
            return session.query(cls).all()

    # 查询所有id
    @classmethod
    def getAllIds(cls, session):
        return session.query(cls.id).all()

    # 根据名字查找
    @classmethod
    def getByName(cls, name, session):
        return cls.getByKV("name", name, session)
        # return session.query(cls).filter(cls.name == name).all()

    # 从偏移量查找
    @classmethod
    def getByFromId(cls, from_id, session, **kw):
        if kw.get("where"):
            condition = list()
            for k, v in dict(kw.get("where")).items():
                condition += [getattr(cls, k) == v]
            if condition:
                return session.query(cls).filter(cls.id > from_id).filter(*condition).limit(kw.get("limit", 10)).all()
        return session.query(cls).filter(cls.id > from_id).limit(kw.get("limit", 10)).all()

    # 从偏移量查找，对ID取模
    @classmethod
    def getByFromIdAndMod(cls, from_id, mod, left, session, **kw):
        if kw.get("where"):
            condition = list()
            for k, v in dict(kw.get("where")).items():
                condition += [getattr(cls, k) == v]
            if condition:
                return session.query(cls).filter(cls.id > from_id, cls.id % mod == left).filter(*condition).limit(kw.get("limit", 10)).all()
        return session.query(cls).filter(cls.id > from_id, cls.id % mod == left).limit(kw.get("limit", 10)).all()

    # 从偏移量查找部分字段，对ID取
    @classmethod
    def getColumsByFromIdAndMod(cls, Colums, from_id, mod, left, session, **kw):
        cls_colum = []
        for Colum in Colums:
            cls_colum.append(getattr(cls, Colum))
        return session.query(tuple(cls_colum)).filter(cls.id > from_id, cls.id % mod == left).limit(kw.get("limit", 10)).all()

    # 更新id
    @classmethod
    def updateId(cls, _id, to_id, session):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set id = " + str(to_id) + " WHERE id = '" + str(_id) + "' LIMIT 1")
        session.commit()
        return res

    # 更新字段
    @classmethod
    def updateKV(cls, _id, k, v, session):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set " + str(k) + " = '" + str(v) + "' WHERE id = '" + str(_id) + "' LIMIT 1")
        session.commit()
        return res

    # 更新字段
    @classmethod
    def updateKVByCondition(cls, cond_k, cond_v, value_k, value_v, session, limit=1):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set " + str(value_k) + " = '" + str(value_v) + "' WHERE " + cond_k + " = '" + str(cond_v) + "' LIMIT " + str(limit))
        session.commit()
        return res

    # 根据字段值获取ID
    @classmethod
    def getIdByKV(cls, k, v, session, **kw):
        return session.query(cls.id).filter(getattr(cls, k) == v).order_by("id").limit(kw.get("limit", 1)).all()

    # 根据多个字段值获取ID
    @classmethod
    def getIdByCondition(cls, d, session, **kw):
        condition = list()
        for k, v in dict(d).items():
            condition += [getattr(cls, k) == v]
        return session.query(cls.id).filter(*condition).order_by("id").limit(kw.get("limit", 1)).all()

    # 根据字段值获取信息
    @classmethod
    def getByKV(cls, k, v, session, **kw):
        return session.query(cls).filter(getattr(cls, k) == v).order_by("id").limit(kw.get("limit", 10)).all()

    # 根据字段值获取信息
    @classmethod
    def getByCondition(cls, d, session, relation="and", **kw):
        condition = list()
        for k, v in dict(d).items():
            condition += [getattr(cls, k) == v]
        if relation == "and":
            return session.query(cls).filter(*condition).order_by("id").limit(kw.get("limit", 10)).all()
        elif relation == "or":
            return session.query(cls).filter(or_(*condition)).order_by("id").limit(kw.get("limit", 10)).all()
        else:
            return None

    # 查询总条数
    @classmethod
    def getCount(cls, session):
        return session.query(func.count(cls.id)).scalar()

    # 获取TopN
    @classmethod
    def getTopN(cls, order, session, **kwargs):
        return session.query(cls).order_by(order).limit(kwargs.get("limit", 10)).all()

    # TODO:Test
    @classmethod
    def getKRegion(cls, k, start_v, end_v, session, **kwargs):
        return session.query(cls).filter(getattr(cls, k) >= start_v, getattr(cls, k) < end_v).order_by(kwargs.get("order", "id")).limit(kwargs.get("limit", 10)).all()

    # ++
    @classmethod
    def increaseKVById(cls, id_, k, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k): getattr(cls, k) + kwargs.get("incr", 1)
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKVById(cls, id_, k, v, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k): func.concat(getattr(cls, k), v)
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKK2KById(cls, id_, k1, k2, k_value, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k_value): func.concat(getattr(cls, k1), func.concat(getattr(cls, k2)))
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKV2KById(cls, id_, k, v, k_value, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k_value): func.concat(getattr(cls, k), v)
        })
        session.commit()
        return res


def createInitFunction(cls):
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(cls):
        if not (str(k).startswith("_") or isinstance(getattr(cls, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))

==> BaseModel-20190527.1004.tar.gz: BaseModel-20190527.1004/BaseModel/BaseModel.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/14 14:18
# @Author   : Peter
# @Site       : Model基类
# @File        : BaseModel.py
# @Software: PyCharm
from sqlalchemy import func, or_, distinct


class BaseModel:

    # 更新model
    @staticmethod
    def mergeModel(model, session, commit=True):
        session.merge(model)
        if commit:
            session.commit()

    # 添加model
    @staticmethod
    def addModel(model, session, commit=True):
        session.add(model)
        if commit:
            session.commit()
    
    # insert & update
    @staticmethod
    def saveModel(model, session, commit=True):
        session.save(model)
        if commit:
            session.commit()

    # query
    @staticmethod
    def query(sql, session, *args, **kwargs):
        return session.execute(sql)

    # 通过id查询
    @classmethod
    def get(cls, _id, session):
        return session.query(cls).get(_id)

    # 查询所有
    @classmethod
    def getAll(cls, session):
        return session.query(cls).all()

    @classmethod
    def getDistinctK(cls, k, session, **kw):
        if kw.get("columns"):
            cls_colum = []
            for Colum in kw.get("columns", []):
                cls_colum.append(getattr(cls, Colum))
            return session.query(*cls_colum).distinct(getattr(cls, k)).order_by(kw.get("order", k)).all()
        else:
            return session.query(distinct(getattr(cls, k))).order_by(kw.get("order", k)).all()

    # 查询所有
    @classmethod
    def getAllByCondition(cls, cond, session, relation="and", **kw):
        condition = list()
        for k, v in dict(cond).items():
            condition += [getattr(cls, k) == v]
        if relation == "and":
            return session.query(cls).filter(*condition).order_by("id").all()
        elif relation == "or":
            return session.query(cls).filter(or_(*condition)).order_by("id").all()
        else:
            return session.query(cls).all()

    # 查询所有id
    @classmethod
    def getAllIds(cls, session):
        return session.query(cls.id).all()

    # 根据名字查找
    @classmethod
    def getByName(cls, name, session):
        return cls.getByKV("name", name, session)
        # return session.query(cls).filter(cls.name == name).all()

    # 从偏移量查找
    @classmethod
    def getByFromId(cls, from_id, session, **kw):
        if kw.get("where"):
            condition = list()
            for k, v in dict(kw.get("where")).items():
                condition += [getattr(cls, k) == v]
            if condition:
                return session.query(cls).filter(cls.id > from_id).filter(*condition).limit(kw.get("limit", 10)).all()
        return session.query(cls).filter(cls.id > from_id).limit(kw.get("limit", 10)).all()

    # 从偏移量查找，对ID取模
    @classmethod
    def getByFromIdAndMod(cls, from_id, mod, left, session, **kw):
        if kw.get("where"):
            condition = list()
            for k, v in dict(kw.get("where")).items():
                condition += [getattr(cls, k) == v]
            if condition:
                return session.query(cls).filter(cls.id > from_id, cls.id % mod == left).filter(*condition).limit(kw.get("limit", 10)).all()
        return session.query(cls).filter(cls.id > from_id, cls.id % mod == left).limit(kw.get("limit", 10)).all()

    # 从偏移量查找部分字段，对ID取
    @classmethod
    def getColumsByFromIdAndMod(cls, Colums, from_id, mod, left, session, **kw):
        cls_colum = []
        for Colum in Colums:
            cls_colum.append(getattr(cls, Colum))
        return session.query(tuple(cls_colum)).filter(cls.id > from_id, cls.id % mod == left).limit(kw.get("limit", 10)).all()

    # 更新id
    @classmethod
    def updateId(cls, _id, to_id, session):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set id = " + str(to_id) + " WHERE id = '" + str(_id) + "' LIMIT 1")
        session.commit()
        return res

    # 更新字段
    @classmethod
    def updateKV(cls, _id, k, v, session):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set " + str(k) + " = '" + str(v) + "' WHERE id = '" + str(_id) + "' LIMIT 1")
        session.commit()
        return res

    # 更新字段
    @classmethod
    def updateKVByCondition(cls, cond_k, cond_v, value_k, value_v, session, limit=1):
        res = session.execute("UPDATE " + str(cls.__tablename__) + " set " + str(value_k) + " = '" + str(value_v) + "' WHERE " + cond_k + " = '" + str(cond_v) + "' LIMIT " + str(limit))
        session.commit()
        return res

    # 根据字段值获取ID
    @classmethod
    def getIdByKV(cls, k, v, session, **kw):
        return session.query(cls.id).filter(getattr(cls, k) == v).order_by("id").limit(kw.get("limit", 1)).all()

    # 根据多个字段值获取ID
    @classmethod
    def getIdByCondition(cls, d, session, **kw):
        condition = list()
        for k, v in dict(d).items():
            condition += [getattr(cls, k) == v]
        return session.query(cls.id).filter(*condition).order_by("id").limit(kw.get("limit", 1)).all()

    # 根据字段值获取信息
    @classmethod
    def getByKV(cls, k, v, session, **kw):
        return session.query(cls).filter(getattr(cls, k) == v).order_by("id").limit(kw.get("limit", 10)).all()

    # 根据字段值获取信息
    @classmethod
    def getByCondition(cls, d, session, relation="and", **kw):
        condition = list()
        for k, v in dict(d).items():
            condition += [getattr(cls, k) == v]
        if relation == "and":
            return session.query(cls).filter(*condition).order_by("id").limit(kw.get("limit", 10)).all()
        elif relation == "or":
            return session.query(cls).filter(or_(*condition)).order_by("id").limit(kw.get("limit", 10)).all()
        else:
            return None

    # 查询总条数
    @classmethod
    def getCount(cls, session):
        return session.query(func.count(cls.id)).scalar()

    # 获取TopN
    @classmethod
    def getTopN(cls, order, session, **kwargs):
        return session.query(cls).order_by(order).limit(kwargs.get("limit", 10)).all()

    # TODO:Test
    @classmethod
    def getKRegion(cls, k, start_v, end_v, session, **kwargs):
        return session.query(cls).filter(getattr(cls, k) >= start_v, getattr(cls, k) < end_v).order_by(kwargs.get("order", "id")).limit(kwargs.get("limit", 10)).all()

    # ++
    @classmethod
    def increaseKVById(cls, id_, k, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k): getattr(cls, k) + kwargs.get("incr", 1)
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKVById(cls, id_, k, v, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k): func.concat(getattr(cls, k), v)
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKK2KById(cls, id_, k1, k2, k_value, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k_value): func.concat(getattr(cls, k1), func.concat(getattr(cls, k2)))
        })
        session.commit()
        return res

    # concat
    @classmethod
    def concatKV2KById(cls, id_, k, v, k_value, session, **kwargs):
        res = session.query(cls).filter(cls.id == id_).update({
            getattr(cls, k_value): func.concat(getattr(cls, k), v)
        })
        session.commit()
        return res


def createInitFunction(cls):
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(cls):
        if not (str(k).startswith("_") or isinstance(getattr(cls, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))
